library(Rssa)
library(polynom)
library(pracma)

ssa.local <- function(F, Z, L = Z%/%2){
  F.local <- list()
  for(i in 1:(length(F)-Z+1)){
    G <- F[seq(i, length.out = Z)]
    F.local[[i]] <- ssa(G,L)
  }
  return(F.local)
}

roots.local_esprit <- function(F.ssa.local,r) 
  sapply(F.ssa.local, function(x) parestimate(x, groups = list(1:r), 
                                  method = 'esprit', solve.method='ls')$roots)

roots.local_esprit_list <- function(F.ssa.local, r) lapply(F.ssa.local, 
                              function(x) parestimate(x, groups = list(1:r), 
                                          method = 'esprit', solve.method = 'ls'))

roots.local_esprit_plot <- function(F, ZZ, rr, LL = ZZ%/%2, t=1){
  for(r in rr)
    for(Z in ZZ){
      F.ssa.local <- ssa.local(F, Z, LL[which(Z == ZZ)])
      F.esprit <- roots.local_esprit_list(F.ssa.local, r)
      lapply(1:length(F.esprit), function(i){
        print(plot(F.esprit[[i]], main = paste('Z=', Z, ' r=', r, ' i=', i, sep='')))
        Sys.sleep(t)
      })      
    }
}

forecast.local <- function(F, R, M, type = 'default', m = 0){
  if(is.vector(R))
    R <- do.call('rbind', replicate(M, R, simplify = FALSE))
  if(type == 'time_series_roots'){
    R <- apply(R, 1, function(x) lrr.time_series(x, m))
    R <- t(R)
  }
  if(type == 'time_series_lrr'){
    R <- apply(R, 1, function(x) lrr.time_series(x, m, 'lrr'))
    R <- t(R)
  }
  if(type == 'lrr.Enlarged'){
    R <- apply(R, 1, function(x) lrr.Enlarged(x, m))
    R <- t(R)
  }
  l <- ncol(R)
  j <- 1
  for(i in seq(length(F)+1, length.out = M)){
    F[i] <- R[j,]%*%F[seq(to = i-1, length.out = l)]
    j <- j+1
  }
  return(tail(F, M))
}

lrr.time_series <- function(x, m, type = 'roots'){
  r <- length(x)
  if(type == 'lrr'){
    class(x) <- 'lrr'
    x <- Rssa::roots(x)
  }
  S <- Re(colSums(outer(x, seq(1, 2*m+1, 1), '^')))
  S.ssa <- ssa(S, m+1)
  return(lrr(S.ssa, group = list(1:r)))
}

lrr.Enlarged <- function(R, L, d = length(R)){
  L <- L+1
  PR <- polynomial(coef = c(-R, 1))
  P <- matrix(0, nrow = L, ncol = L-d)
  for(j in 1:ncol(P))
    P[j:(j+d),j]<-c(-R, 1)
  n <- L-d-1
  cma <- chol(t(P)%*%P)
  H <- chol2inv(cma)%*%c(rep(0, n), 1)
  PH <- polynomial(coef = H)
  Pres <- monic(PR*PH)
  Lrr<- -as.vector(Pres)
  Lrr <- Lrr[-length(Lrr)]
  class(Lrr)<-'lrr'
  return(Lrr)
}

roots.forecast <- function(P, M, z, type = 'last', L = length(P)%/%2, r = 2){
  N <- seq(301, length.out = M)
  if(type == 'last'){
    k <- tail(which(!is.na(P)),1)
    return(rep(P[k], M))
  }
  if(type == 'mean')
    return(rep(mean(P, na.rm = TRUE), M))
  if(type == 'regression'){
    line_P <- lm(P~as.numeric(1:length(P)))
    return(line_P$coefficients[2]*(N-z)+line_P$coefficients[1])
  }
  if(type == 'ssa'){
    if(anyNA(P)){
      P.ssa <- ssa(P, L, force.decompose = FALSE)
      P.reconstruct <- igapfill(P.ssa, groups = list(1:r))
      P.ssa <- ssa(P.reconstruct, L)
    }else
      P.ssa <- ssa(P, L)
    P.forecast <- rforecast(P.ssa, groups = list(1:r), len = M+z)
    return(P.forecast[seq(from = Z%/%2, length.out = M)])
  }
}

pair_roots<-function(R){
  R[1,]<-R[1,order(Arg(R[1,]),decreasing=TRUE)]
  for(i in 2:nrow(R))
    R[i,]<-pair(R[i-1,],R[i,])
  return(R)
}

lrr.cos <- function(arg, mod) cbind(-mod^2, 2*mod*cos(arg))

lrr.combine.cos <- function(R){
  P <- lapply(seq(1, length(R), 2), function(i) polynomial(c(-R[i:(i+1)], 1)))
  Prod <- 1
  for(i in 1:length(P))
    Prod <- Prod*P[[i]]
  Lrr <- -as.vector(Prod)
  Lrr <- Lrr[-length(Lrr)]
  class(Lrr) <- 'lrr'
  return(Lrr)
}

arg_mod.forecast <- function(F.roots_arg, F.roots_mod, M, Z, type_arg = 'regression', type_mod = 'mean', L_ssa = rep((nrow(F.roots_arg))%/%2, length(type_arg)), r_ssa = 0){
  if(is.vector(F.roots_arg)){
    F.roots_arg <- matrix(F.roots_arg, ncol=1)
    F.roots_mod <- matrix(F.roots_mod, ncol=1)
  }
  forecast_arg <- forecast_mod <- matrix(nrow = M, ncol = ncol(F.roots_arg))
  for(j in 1:ncol(F.roots_arg)){
    forecast_arg[,j] <- roots.forecast(F.roots_arg[,j], M, Z%/%2, type_arg[j], L_ssa[j], r_ssa[j])
    forecast_mod[,j] <- roots.forecast(F.roots_mod[,j], M, Z%/%2, type_mod[j], L_ssa[j], r_ssa[j])
  }
  return(list(Arg = forecast_arg, Mod = forecast_mod))
}

lrr.count<-function(forecast_arg,forecast_mod){#,M,z,type_arg='regression',type_mod='mean',L_ssa=rep((length(F.roots_arg)-2*z+1)%/%2,length(type_arg)),r_ssa=0){
  if(is.vector(forecast_arg)){
    forecast_arg<-matrix(forecast_arg,ncol=1)
    forecast_mod<-matrix(forecast_mod,ncol=1)
  }
  R<-matrix(nrow=nrow(forecast_arg),ncol=0)
  for(j in 1:ncol(forecast_arg))
    R<-cbind(R,lrr.cos(forecast_arg[,j],forecast_mod[,j]))
  R<-apply(R,1,lrr.combine.cos)
  return(t(R))
}

lrr.combine <- function(R){
  P <- lapply(R, function(x) polynomial(c(-x, 1)))
  Prod <- P[[1]]
  for(i in 2:length(P))
    Prod <- Prod*P[[i]]
  Lrr <- as.vector(Prod)
  Lrr <- Lrr[-length(Lrr)]
  class(Lrr) <- 'lrr'
  return(Lrr)
}


roots.get_arg_mod <- function(F.roots){
  if(is.vector(F.roots))
    F.roots <- matrix(F.roots, ncol = 1)
  F.roots <- t(sapply(1:nrow(F.roots), function(i) F.roots[i, order(Arg(F.roots[i,]), decreasing = TRUE)]))
  F.roots <- F.roots[, 1:(ncol(F.roots)%/%2), drop = FALSE]
  F.roots <- pair_roots(F.roots)
  real_roots_index <- apply(F.roots, 1:2, function(x) if(all.equal(Im(x), 0) == TRUE)return(TRUE)else return(FALSE))
  F.roots[real_roots_index] <- NA
  F.roots_arg <- apply(F.roots, 2, Arg)
  F.roots_mod <- apply(F.roots, 2, Mod)
  return(list(Arg = F.roots_arg, Mod = F.roots_mod))
}

lrr.count_sample <- function(sample, Z, L, r, M, type_arg = 'regression', type_mod = 'mean', L_ssa = (length(F)-Z+1)%/%2, r_ssa = 0, type_forecast = 'default', m = 0){
  F.ssa <- lapply(sample, function(x) ssa.local(x, Z, L))
  F.roots <- lapply(F.ssa, function(x) t(roots.local_esprit(x, r)))
  F.roots_arg_mod <- lapply(F.roots, roots.get_arg_mod)
  n <- length(sample)
  F.arg_mod_forecast <- lapply(1:n, function(i) arg_mod.forecast(F.roots_arg_mod[[i]]$Arg, F.roots_arg_mod[[i]]$Mod, M, Z, type_arg, type_mod, L_ssa, r_ssa))
  R <- list()
  for(i in 1:length(sample)){
    R[[i]] <- lrr.count(F.arg_mod_forecast[[i]]$Arg, F.arg_mod_forecast[[i]]$Mod)
  }
  return(R)
}

forecast.local_sample <- function(sample, R, type_forecast = 'default', m = 0){
  F.forecast <- list()
  for(i in 1:length(sample))
    F.forecast[[i]] <- forecast.local(sample[[i]], R[[i]], M, type_forecast, m)
  return(F.forecast)
}

MSE.count <- function(F.forecast, F, A = 1) mean(((F.forecast-F)/A)^2)

lrr.enlarging_sample <- function(sample, Z, L, r, M, mm, type_arg = 'regression', type_mod = 'mean', L_ssa = rep((length(sample[[1]])-Z+1)%/%2, length(type_arg)), r_ssa = 0, type_forecast = 'default', S_current){
  RMSEs <- vector(length = max(mm))
  R <- lrr.count_sample(sample, Z, L, r, M, type_arg, type_mod, L_ssa, r_ssa)
  for(m in mm){
    sample.forecast <- forecast.local_sample(sample, R, type_forecast, m = m)
    RMSEs[m] <- sqrt(mean(sapply(sample.forecast, function(x) MSE.count(x, S_current))))
  }
  return(RMSEs)
}

pair <- function(R1, R2){
  P <- perms(1:length(R1))
  Norms <- apply(P, 1, function(p) norm(abs(R1-R2[p]), '2'))
  return(R2[P[which.min(Norms),]])
}

create_sample <- function(S, N, sigma = 1){
  Sample <- lapply(N, function(x) S+sigma*x)
  return(Sample)
}