rm(list = ls())
setwd(dirname(rstudioapi::getActiveDocumentContext()$path))
getwd()
source('functions.R')
set.seed(1001)

path <- 'fig/'
if(!file.exists(path)) dir.create(path)
w <- 10 / 1.14
h <- 5.75 / 1.3 / 1.14
bg <- 'white'
skip <- 221:300#1:300

sigma <- 0

signals <-
  c('cos(n^2).R',
    'cossin(n).R',
    'cos(n^2)+cossin(n).R',
    'cossin(n)+cossin(n).R')

main_fig <- function(number) {
  source(signals[number])
  main <- paste0(main, sigma, 'm', m)
  S <- s(n)
  N <- seq(1 + tail(n, 1), length.out = M)
  S_current <- s(N)
  pdf(paste0(path, main, '.pdf'),
      width = w,
      height = h)
  plot(S,
       type = 'l',
       xlab = 'Time',
       ylab = '')
  dev.off()
  if (r >= 4) {
    S1 <- s1(n)
    S2 <- s2(n)
    pdf(paste0(path, main, '1.pdf'),
        width = w,
        height = h)
    plot(S1,
         type = 'l',
         xlab = 'Time',
         ylab = '')
    dev.off()
    pdf(paste0(path, main, '2.pdf'),
        width = w,
        height = h)
    plot(S2,
         type = 'l',
         xlab = 'Time',
         ylab = '')
    dev.off()
    S1_current <- s1(N)
    S2_current <- s2(N)
  }
  
  #plot esprit roots estimations
  #roots.local_esprit_plot(S,Z,r,L,0.1)
  
  RMSE_0 <- sqrt(MSE.count(S_current, 0))
  omega_th <- lapply(omega_ins, function(f)
    function(x)
      f(x + Z %/% 2))
  pdf(paste0(path, main, '_omega_ins.pdf'),
      width = w,
      height = h)
  plot(
    omega_th[[1]],
    xlim = c(0, tail(n, 1)),
    xlab = 'Time',
    ylab = expression(omega[ins]),
    type = 'l',
  )#ylim=c(0,0.3))
  if (length(omega_th) > 1)
    for (i in 2:length(omega_th)) {
      pdf(paste0(path, main, '_omega_ins', i, '.pdf'),
          width = w,
          height = h)
      plot(
        omega_th[[i]],
        xlim = c(0, tail(n, 1)),
        xlab = 'Index of interval',
        ylab = expression(omega[ins]),
        type = 'l'
      )
      o <- Vectorize(omega_th[[i]], 'x')
      curve(o, add = TRUE)
    }
  dev.off()
  
  for (i in 1:length(omega_th)) {
    pdf(paste0(path, main, '_omega_ins', i, '.pdf'),
        width = w,
        height = h)
    plot(
      omega_th[[i]],
      xlim = c(0, tail(n, 1)),
      xlab = 'Index of interval',
      ylab = expression(omega[ins]),
      type = 'l'
    )
    dev.off()
  }
  
  #count arg and mod of esprit roots
  X <- S + sigma * rnorm(length(S))
  S.ssa <- ssa.local(X, Z, L)
  S.roots <- t(roots.local_esprit(S.ssa, r))
  S.roots_arg_mod <- roots.get_arg_mod(S.roots)
  
  
  for (i in 1:length(omega_th)) {
    pdf(paste0(path, main, '_omega_est', i, '.pdf'),
        width = w,
        height = h)
    plot(
      S.roots_arg_mod$Arg[, i] / (2 * pi),
      ylab = expression(omega ^ (i)),
      xlab = 'Index of interval',
      type = 'b',
      ylim = range(c(
        S.roots_arg_mod$Arg[, i] / (2 * pi), omega_th[[i]](n)
      ))
    )
    o <- Vectorize(omega_th[[i]], 'x')
    curve(o, add = TRUE)
    legend(
      'topleft',
      c('Estimation', 'Theoretical'),
      pch = c(21, NA),
      pt.bg = 'white',
      lty = c(1, 1)
    )
    dev.off()
  }
  
  for (i in 1:length(omega_th)) {
    pdf(paste0(path, main, '_mod_est', i, '.pdf'),
        width = w,
        height = h)
    plot(
      S.roots_arg_mod$Mod[, i],
      type = 'b',
      xlab = 'Index of interval',
      ylab = expression(abs(mu ^ (i))),
      ylim = c(0.97, 1.03)
    )
    abline(1, 0)
    legend(
      'topleft',
      c('Estimation', 'Theoretical'),
      pch = c(21, NA),
      pt.bg = 'white',
      lty = c(1, 1)
    )
    dev.off()
  }
  
  #forecast of arg and mod of esprit roots
  source('functions.R')
  
  S.arg_mod_forecast <-
    arg_mod.forecast(
      S.roots_arg_mod$Arg,
      S.roots_arg_mod$Mod,
      M,
      Z %/% 2,
      rep('last', length(omega_ins)),
      rep('last', length(omega_ins))
    )
  
  #forecast using last local interval
  R <- lrr.count(S.arg_mod_forecast$Arg, S.arg_mod_forecast$Mod)
  S_last <- forecast.local(X, R, M, 'lrr.Enlarged', m = m)
  RMSE_last <- sqrt(MSE.count(S_last, S_current))
  pch_type = 20
  pdf(paste0(path, main, '_last.pdf'),
      width = w,
      height = h)
  plot(
    c(skip, N),
    c(X[skip], S_current),
    type = 'l',
    ylab = '',
    xlab = 'Time',
    ylim = range(c(X[skip], S_current, S_last))
  )
  lines(c(skip, N), c(rep(NA, length(skip)), S_last), type = 'b', pch =
          pch_type)
  legend(
    'topleft',
    c('Signal', 'Forecast'),
    lty = c(1, 1),
    pch = c(NA, pch_type),
    bg = bg
  )
  dev.off()
  
  #forecast of module and argument using algorithm 2.2
  l_type = 1
  pch_type = 21
  col_type = 'red'
  S.arg_mod_forecast <-
    arg_mod.forecast(S.roots_arg_mod$Arg,
                     S.roots_arg_mod$Mod,
                     M,
                     Z,
                     type_arg,
                     type_mod,
                     r_ssa = r_ssa)
  for (i in 1:length(omega_th)) {
    pdf(
      paste0(path, main, '_omega_forecast', i, '.pdf'),
      width = w / 2,
      height = h / 1.2
    )
    plot(
      c(
        S.roots_arg_mod$Arg[, i] / (2 * pi),
        rep(NA, Z %/% 2),
        S.arg_mod_forecast$Arg[, i] / (2 * pi)
      ),
      ylim = range(
        c(
          S.roots_arg_mod$Arg[, i] / (2 * pi),
          S.arg_mod_forecast$Arg[, i] / (2 * pi),
          omega_th[[i]](n),
          omega_th[[i]](N)
        )
      ),
      pch = c(rep(21, length(
        S.roots_arg_mod$Arg[, i]
      )), rep(NA, Z %/% 2), rep(
        pch_type, length(S.arg_mod_forecast$Arg[, i])
      )),
      type = 'b',
      ylab = expression(omega ^ (i)),
      xlab = 'Index of interval',
      col = c(
        rep('black', length(S.roots_arg_mod$Arg[, i])),
        rep(NA, Z %/% 2),
        rep(col_type, length(S.arg_mod_forecast$Arg[, i]))
      )
    )#),ylim=c(min(omega_th[[i]](n),S.arg_),max(omega_th[[i]](n))))
    legend(
      'topleft',
      c('Estimation', 'Theoretical', 'Forecast'),
      pch = c(21, NA, pch_type),
      pt.bg = 'white',
      lty = c(1, 1, 1),
      col = c('black', 'black', col_type)
    )
    o <- Vectorize(omega_th[[i]], 'x')
    curve(o, add = TRUE)
    dev.off()
  }
  
  #Forecast using algorithm
  R <- lrr.count(S.arg_mod_forecast$Arg, S.arg_mod_forecast$Mod)
  S_alg <- forecast.local(X, R, M, 'lrr.Enlarged', m = m)
  RMSE_alg <- sqrt(MSE.count(S_current, S_alg))
  
  pch_type = 20
  pdf(paste0(path, main, '_alg.pdf'),
      width = w,
      height = h)
  plot(
    c(skip, N),
    c(X[skip], S_current),
    type = 'l',
    ylab = '',
    xlab = 'Time',
    ylim = range(c(S_current, S_alg, X[skip]))
  )
  lines(c(skip, N), c(rep(NA, length(skip)), S_alg), type = 'b', pch = pch_type)
  legend(
    'topleft',
    c('Signal', 'Forecast'),
    lty = c(1, 1),
    pch = c(NA, pch_type),
    bg = bg
  )
  dev.off()
  
  #Forecast using enlarged LRR
  S_Enlarged <- forecast.local(X, R, M, 'lrr.Enlarged', m = m)
  RMSE_Enlarged <- sqrt(MSE.count(S_Enlarged, S_current))
  pdf(paste0(path, main, '_Enlarged.pdf'),
      width = w,
      height = h)
  plot(
    c(skip, N),
    c(X[skip], S_current),
    type = 'l',
    ylab = '',
    xlab = 'Time',
    ylim = range(c(X[skip], S_current, S_Enlarged))
  )
  lines(c(skip, N),
        c(rep(NA, length(skip)), S_Enlarged),
        type = 'b',
        pch = pch_type)
  legend(
    'topleft',
    c('Signal', 'Forecast'),
    lty = c(1, 1),
    pch = c(NA, pch_type),
    bg = bg
  )
  dev.off()
  
  print(signals[number])
  print(RMSE_0)
  print(RMSE_last)
  print(RMSE_alg)
  print(RMSE_Enlarged)
}

for (i in 1:4){
  set.seed(1001)
  main_fig(i)
}
