rm(list = ls())
setwd(dirname(rstudioapi::getActiveDocumentContext()$path))
getwd()
source('functions.R')
set.seed(1001)

path <- 'fig/'
if(!file.exists(path)) dir.create(path)
w <- 10 / 1.14
h <- 5.75 / 1.3 / 1.14
bg <- 'white'
skip <- 221:300#1:300
sigma <- 0.25
size <- 1 # 100 - time consuming

signals <-
  c('cos(n^2).R',
    'cossin(n).R',
    'cos(n^2)+cossin(n).R',
    'cossin(n)+cossin(n).R')
main_tab <- function(number) {
  source(signals[number])
  main <- paste0(main, sigma)
  
  S = s(n)
  N <- seq(1 + tail(n, 1), length.out = M)
  S_current <- s(N)
  Noise <- replicate(size, rnorm(length(n)), simplify = FALSE)
  sample_S <- create_sample(S, Noise, sigma)
  
  #count RMSEs for each m (LRR size) in vector mm
  RMSE_0 <- sqrt(MSE.count(S_current, 0))
  RMSEs <-
    matrix(c(RMSE_0, sqrt(RMSE_0 ^ 2 + sigma ^ 2)), nrow = 2, ncol = 5) #table of RMSEs
  RMSEs_last_clear <-
    lrr.enlarging_sample(
      list(S),
      Z,
      L,
      r,
      M,
      mm,
      type_forecast = 'lrr.Enlarged',
      type_arg = type_arg_last,
      type_mod = type_mod_last,
      r_ssa = r_ssa,
      S_current = S_current
    )
  i_last <- which.min(RMSEs_last_clear[mm])
  RMSEs[1, 2:3] <- rev(c(mm[i_last], RMSEs_last_clear[mm[i_last]]))

  RMSEs_alg_clear <-
    lrr.enlarging_sample(
      list(S),
      Z,
      L,
      r,
      M,
      mm,
      type_forecast = 'lrr.Enlarged',
      type_arg = type_arg,
      type_mod = type_mod,
      r_ssa = r_ssa,
      S_current = S_current
    )
  i_alg <- which.min(RMSEs_alg_clear[mm])
  RMSEs[1, 4:5] <- rev(c(mm[i_alg], RMSEs_alg_clear[mm[i_alg]]))
  
  print(signals[number])

  RMSEs_last_res <-
    lrr.enlarging_sample(
      sample_S,
      Z,
      L,
      r,
      M,
      mm,
      type_forecast = 'lrr.Enlarged',
      type_arg = type_arg_last,
      type_mod = type_mod_last,
      r_ssa = r_ssa,
      S_current = S_current
    )
  i_last <- which.min(RMSEs_last_res[mm])
  RMSEs[2, 2:3] <- rev(c(mm[i_last], RMSEs_last_res[mm[i_last]]))

  RMSEs_alg_res <-
    lrr.enlarging_sample(
      sample_S,
      Z,
      L,
      r,
      M,
      mm,
      type_forecast = 'lrr.Enlarged',
      type_arg = type_arg,
      type_mod = type_mod,
      r_ssa = r_ssa,
      S_current = S_current
    )
  i_alg <- which.min(RMSEs_alg_res[mm])
  RMSEs[2, 4:5] <- rev(c(mm[i_alg], RMSEs_alg_res[mm[i_alg]]))
  
  pdf(paste0(path, main, '_mm.pdf'))
  rownames(RMSEs) <- c(0, sigma)
  pch_type = 17
  plot(
    mm,
    RMSEs_alg_res[mm],
    type = 'b',
    xlab = 'LRR length',
    ylab = 'RMSE',
    ylim = c(min(c(
      RMSEs_alg_res[mm], RMSEs_last_res[mm]
    )), max(c(
      RMSEs_alg_res[mm], RMSEs_last_res[mm]
    ))),
    pch = 20
  )
  lines(mm, RMSEs_last_res[mm], pch = pch_type, type = 'b')
  lines(mm, rep(sqrt(mean(S_current ^ 2) + sigma ^ 2), length(mm)), type =
          'l')
  legend(
    'bottomright',
    c('const', 'last', 'alg'),
    pch = c(NA, pch_type, 20),
    pt.bg = 'white',
    lty = c(1, 1, 1)
  )
  dev.off()
  
  #print('-----------')
  print(RMSEs)
  saveRDS(RMSEs, paste0(number, '.RData'))
}


for (i in 1:4) {
  set.seed(1001)
  main_tab(i)
}